function setup() {
	let content = document.querySelector(".content");
	content.addEventListener("click", () => {
		document.querySelector("#menu-toggle").checked = false;
	});
}

function convertLinks() {
	let links = document.querySelectorAll("a");
	links.forEach(el => {
		el.removeEventListener("click", linkClick);
		el.addEventListener("click", linkClick);
	});
}

async function linkClick(event) {
	let a = event.currentTarget;
	if (a.origin == location.origin) {
		if (a.hash != "" && a.pathname == location.pathname) return;
		event.preventDefault();
		changePage(a.href);
	} else {
		a.target = "_blank";
	}
}

async function changePage(url, addHistory = true) {
	document.querySelector(".content").classList.add("hidden");
	let page = await downloadPage(url);
	let data = parsePage(page);
	if (data) {
		updatePage(url, data, addHistory);
	}
}

function updatePage(url, d, addHistory) {
	document.title = d.title;
	if (addHistory) history.pushState({}, d.title, url);
	if (d.sidebar.classList.contains("hidden")) {
		document.querySelector(".sidebar").classList.add("hidden");
	}
	updateHTML("nav .right ul", d.links);
	setTimeout(() => {
		updateHTML(".sidebar", d.sidebar);
		document.querySelector(".markdown-body").innerHTML = d.content.innerHTML;
		convertLinks();
		document.querySelector("head").innerHTML = d.head.innerHTML;
		document.querySelector("html").scrollTo(0, 0);
		document.querySelector(".content").classList.remove("hidden");
	}, 300);
}

function parsePage(html) {
	let parser = new DOMParser().parseFromString(html ?? "", "text/html");
	let content = parser.querySelector(".markdown-body");
	let links = parser.querySelector("nav .right ul");
	let sidebar = parser.querySelector(".sidebar");
	let head = parser.querySelector("head");
	if (!content || !links || !sidebar || !head) return null;
	let titleEl = head.querySelector("title");
	let title = titleEl ? titleEl.innerText : "";
	return {title, content, links, sidebar, head};
}

async function downloadPage(url) {
	return new Promise(function(resolve, reject) {
		var xhr = new XMLHttpRequest();
		xhr.onreadystatechange = function() {
			if (this.readyState !== 4) return;
			if (!this.responseText) reject(this);
			try {
				resolve(this.responseText);
			} catch(e) {
				reject();
			}
		};
		xhr.open("GET", url, true);
		xhr.send();
	});
}

function updateHTML(selector, newEl) {

	let origEl = document.querySelector(selector);
	if (!origEl) return;

	function updateElement(el, newEl) {

		// Replace element if it has different tagName
		if (el.tagName != newEl.tagName) {
			el.replaceWith(newEl);
			return;
		}
		// Change element's innerHTML, if there is different children count or there is non-empty text node
		if (el.children.length != newEl.children.length || getTextNodeCount(newEl.childNodes) || getTextNodeCount(el.childNodes)) {
			el.innerHTML = newEl.innerHTML;
			return;
		}
		// Add all attributes of the new element
		for (const { name, value } of newEl.attributes) {
			el.setAttribute(name, value);
		}
		// Remove no longer present attributes
		for (const { name } of el.attributes) {
			if (!newEl.getAttribute(name)) el.removeAttribute(name);
		}
		// Update all childrens
		for (let i=0; i<el.children.length; i++) {
			updateElement(el.children[i], newEl.children[i]);
		}

	}

	updateElement(origEl, newEl);

}

function getTextNodeCount(nodes) {
	return [...nodes].map(
		e => (e.nodeType == 3 && e.textContent != undefined && e.textContent.trim() != "") ? 1 : 0
	).reduce((p, c) => p + c, 0);
}

window.addEventListener("popstate", e => {
	if (e.state === null) return;
	changePage(window.location.href, false);
});

window.addEventListener("load", () => {
	convertLinks();
	setup();
});